package dbtypes

import (
	"errors"

	"git.andr3h3nriqu3s.com/andr3/fyp/logic/db"
)

const (
	FAILED_TRAINING           = -4
	FAILED_PREPARING_TRAINING = -3
	FAILED_PREPARING_ZIP_FILE = -2
	FAILED_PREPARING          = -1

	PREPARING               = 1
	CONFIRM_PRE_TRAINING    = 2
	PREPARING_ZIP_FILE      = 3
	TRAINING                = 4
	READY                   = 5
	READY_ALTERATION        = 6
	READY_ALTERATION_FAILED = -6

	READY_RETRAIN        = 7
	READY_RETRAIN_FAILED = -7
)

type ModelDefinitionStatus int

type LayerType int

const (
	LAYER_INPUT        LayerType = 1
	LAYER_DENSE                  = 2
	LAYER_FLATTEN                = 3
	LAYER_SIMPLE_BLOCK           = 4
)

const (
	MODEL_DEFINITION_STATUS_CANCELD_TRAINING ModelDefinitionStatus = -4
	MODEL_DEFINITION_STATUS_FAILED_TRAINING                        = -3
	MODEL_DEFINITION_STATUS_PRE_INIT                               = 1
	MODEL_DEFINITION_STATUS_INIT                                   = 2
	MODEL_DEFINITION_STATUS_TRAINING                               = 3
	MODEL_DEFINITION_STATUS_PAUSED_TRAINING                        = 6
	MODEL_DEFINITION_STATUS_TRANIED                                = 4
	MODEL_DEFINITION_STATUS_READY                                  = 5
)

type ModelClassStatus int

const (
	MODEL_CLASS_STATUS_TO_TRAIN ModelClassStatus = 1
	MODEL_CLASS_STATUS_TRAINING                  = 2
	MODEL_CLASS_STATUS_TRAINED                   = 3
)

type ModelHeadStatus int

const (
	MODEL_HEAD_STATUS_PRE_INIT ModelHeadStatus = 1
	MODEL_HEAD_STATUS_INIT                     = 2
	MODEL_HEAD_STATUS_TRAINING                 = 3
	MODEL_HEAD_STATUS_TRAINED                  = 4
	MODEL_HEAD_STATUS_READY                    = 5
)

type BaseModel struct {
	Name   string
	Status int
	Id     string

	ModelType    int    `db:"model_type"`
	ImageModeRaw string `db:"color_mode"`
	ImageMode    int    `db:"0"`
	Width        int
	Height       int
	Format       string
	CanTrain     int `db:"can_train"`
}

var ModelNotFoundError = errors.New("Model not found error")

func GetBaseModel(db db.Db, id string) (base *BaseModel, err error) {
	var model BaseModel
	err = GetDBOnce(db, &model, "models where id=$1", id)
	if err != nil {
		return
	}
	model.ImageMode = StringToImageMode(model.ImageModeRaw)
	return &model, nil
}

func (m BaseModel) CanEval() bool {
	if m.Status != READY && m.Status != READY_RETRAIN && m.Status != READY_RETRAIN_FAILED && m.Status != READY_ALTERATION && m.Status != READY_ALTERATION_FAILED {
		return false
	}
	return true
}

func StringToImageMode(colorMode string) int {
	switch colorMode {
	case "greyscale":
		return 1
	case "rgb":
		return 3
	default:
		panic("unkown color mode")
	}
}
